/*
* ae.c
* general purpose arithmetic expression package based on Lua
* this code is hereby placed in the public domain by lhf@tecgraf.puc-rio.br
* 01 Jun 2006 15:26:20
*/

#include <ctype.h>
#include <string.h>
#include "lua.h"
#include "lualib.h"
#include "lauxlib.h"

#ifndef LUA_VERSION_NUM
#include <math.h>

static int math_pow (lua_State *L) {
  lua_pushnumber(L, pow(luaL_checknumber(L, 1), luaL_checknumber(L, 2)));
  return 1;
}
#endif

#include "ae.h"

static lua_State *L=NULL;

void ae_open(int funs)
{
 L=lua_open();
 if (funs)
 {
  lua_pushliteral(L,LUA_MATHLIBNAME);
  lua_pushvalue(L,LUA_GLOBALSINDEX);
  lua_rawset(L,LUA_GLOBALSINDEX);
  luaopen_math(L);
  lua_pushliteral(L,LUA_MATHLIBNAME);
  lua_pushnil(L);
  lua_rawset(L,LUA_GLOBALSINDEX);
  lua_settop(L,0);
 }
#ifndef LUA_VERSION_NUM
 else
 {
  lua_pushliteral(L, "__pow");
  lua_pushcfunction(L, math_pow);
  lua_settable(L, LUA_GLOBALSINDEX);
 }
#endif
}

void ae_close(void)
{
 lua_close(L);
 L=NULL;
}

double ae_set(const char* name, double value)
{
 lua_pushnumber(L,value);
 lua_setglobal(L,name);
 return value;
}

double ae_get(const char* name)
{
 double value;
 lua_getglobal(L,name);
 value=lua_tonumber(L,-1);
 lua_settop(L,0);
 return value;
}

typedef struct LoadS
{
 const char *s[20];
 size_t size[20];
 int i;
} LoadS;

static const char *getS (lua_State *L, void *data, size_t *size)
{
 LoadS *S=(LoadS*) data;
 int i=S->i++;
 *size=S->size[i];
 (void)L;
 return S->s[i];
}

static int dostring(LoadS *S, int n)
{
#if 0
 int i;
 for (i=0; S->s[i]; i++) printf("%.*s",S->size[i],S->s[i]); fputs("\n",stdout);
#endif
 S->i=0;
 if (lua_load(L,getS,S,"=ae") || lua_pcall(L,0,n,0))
 {
  lua_pushstring(L,strchr(lua_tostring(L,-1),' ')+1);
  lua_rawseti(L,LUA_REGISTRYINDEX,0);
  return 1;
 }
 else
  return 0;
}

#define RETURN		" return "
#define FUNCTION	" function "
#define END		" end "

double ae_eval(const char* expression, int* rc)
{
 int c;
 double value;
 LoadS S;
 S.s[0]=RETURN;		S.size[0]=sizeof(RETURN)-1;
 S.s[1]=expression;	S.size[1]=strlen(expression);
 S.s[2]=NULL;		S.size[2]=0;
 if ((c=dostring(&S,1))) value=0.0; else value=lua_tonumber(L,-1);
 if (rc!=NULL) *rc=c;
 lua_settop(L,0);
 return value;
}


const char* ae_fun(const char* expression)
{
 LoadS S;
 const char* s=expression;
 const char* b;
 int i=-1;
 while (isspace(*s)) s++;
 i++; S.s[i]=RETURN;		S.size[i]=sizeof(RETURN)-1;
 i++; S.s[i]="'";		S.size[i]=1;
 b=s;
 while (isalnum(*s)) s++;
 i++; S.s[i]=b;			S.size[i]=s-b;
 i++; S.s[i]="',";		S.size[i]=2;
 i++; S.s[i]=FUNCTION;		S.size[i]=sizeof(FUNCTION)-1;
 b=s;
 while (*s!='=' && *s!=0) s++;
 if (*s==0) return "bad function definition";
 i++; S.s[i]=b;			S.size[i]=s-b;
 i++; S.s[i]=RETURN;		S.size[i]=sizeof(RETURN)-1;
 i++; S.s[i]=s+1;		S.size[i]=strlen(s+1);
 i++; S.s[i]=END;		S.size[i]=sizeof(END)-1;
 i++; S.s[i]=NULL;		S.size[i]=0;
 if (dostring(&S,2))
  return ae_error();
 else
 {
  lua_setglobal(L,lua_tostring(L,-2));
  lua_settop(L,0);
  return NULL;
 }
}

const char* ae_error(void)
{
 const char* s;
 lua_rawgeti(L,LUA_REGISTRYINDEX,0);
 s=lua_tostring(L,-1);
 lua_settop(L,0);
 return s;
}
