/*
* ae.h
* evaluate arithmetic expressions at run time
* Luiz Henrique de Figueiredo <lhf@tecgraf.puc-rio.br>
* 01 Jun 2006 15:05:52
* This code is hereby placed in the public domain.
*/

void		ae_open		(int funs);
void		ae_close	(void);
double		ae_set		(const char* name, double value);
double		ae_get		(const char* name);
double		ae_eval		(const char* expression, int* rc);
const char*	ae_fun		(const char* expression);
const char* 	ae_error	(void);

/*

  ae_open(int funs)
    Opens ae to be used. Call it once before calling the others.
    Use funs=1 to load elementary functions or funs=0 if you don't need them.

  ae_close()
    Closes ae after use.

  ae_set(name,value)
    Sets the value of a variable.
    The value persists until it is set again or ae_close is called.

  ae_get(name)
    Gets the value of a variable.
    Variables that have not been set explicitly by ae_set have value 0.

  ae_eval(expression,&rc)
    Evaluates the given expression and returns its value.
    Returns 0 if there is an error and sets rc to 1.
    It's safe to use &rc==NULL, if you don't care to check errors.
    Use ae_error to get the error message in case of errors.

  ae_fun(expression)
    Defines a function.
    Returns the last error message or NULL if no error occurred.
    The expression should be of the form "f(x,y)=x+2*y", that is, a name,
    followed by the list of parameters, followed by an equal sign, followed
    by an arithmetic expression involving the parameters and possibly other
    variables. Whitespace is of course allowed everywhere.

  ae_error(void)
    Returns the last error message.

*/
